﻿namespace ProjectGasolineTanker.CollectionGenericObjects;

/// <summary>
/// Параметризованный набор объектов
/// </summary>
/// <typeparam name="T">Параметр: ограничение - ссылочный тип</typeparam>
public class MassiveGenericObjects<T> : ICollectionGenericObjects<T>
    where T : class
{
    /// <summary>
    /// Массив объектов, которые храним
    /// </summary>
    private T?[] _collection;

    public int Count => _collection.Length;

    public int SetMaxCount
    {
        set
        {
            if (value > 0)
            {
                if (Count > 0)
                {
                    Array.Resize(ref _collection, value);
                }
                else
                {
                    _collection = new T?[value];
                }
            }
        }
    }


    /// <summary>
    /// Конструктор
    /// </summary>
    public MassiveGenericObjects()
    {
        _collection = Array.Empty<T?>();
    }


    public T? Get(int position)
    {
        // проверка позиции
        if (position >= Count || position < 0)
        {
            return null;
        }
        return _collection[position];
    }
    public int Insert(T obj)
    {
        // вставка в свободное место набора
        for (int i = 0; i < Count; ++i)
        {
            if (_collection[i] == null)
            {
                _collection[i] = obj;
                return i;
            }
        }
        return -1;


    }
    public int Insert(T obj, int position)
    {
        // проверка позиции
        // проверка, что элемент массива по этой позиции пустой, если нет, то
        // ищется свободное место после этой позиции и идет вставка туда, если нет после, ищем до
        // вставка

        if (position >= Count || position < 0)
        {
            return -1;
        }

        if (_collection[position] == null)
        {
            _collection[position] = obj;
            return position;
        }
        else
        {
            for (int i = 1; i < Count; ++i)
            {
                if (_collection[position + i] == null)
                {
                    _collection[position + i] = obj;
                    return position + i;
                }
                for (i = position - 1; i >= 0; i--)
                {
                    if (_collection[i] == null)
                    {
                        _collection[i] = obj;
                        return i;
                    }
                }
            }
        }
        return -1;

    }
    public T? Remove(int position)
    {
        //// проверка позиции
        //// удаление объекта из массива, присвоив элементу массива значение null
       
        if (position >= Count || position < 0) return null;
        T temp = _collection[position];
        _collection[position] = null;
        return temp;
    }
}
