﻿using ProjectGasolineTanker.Entities;

namespace ProjectGasolineTanker.Drawings;

public class DrawingTruck
{/// <summary>
 /// Класс-сущность
 /// </summary>
    public EntityTruck? EntityTruck { get; protected set; }

    /// <summary>
    /// Ширина окна
    /// </summary>
    private int? _pictureWidth;

    /// <summary>
    /// Высота окнаы
    /// </summary>
    private int? _pictureHeight;

    /// <summary>
    /// Левая координата прорисовки бензовоза
    /// </summary>
    protected int? _startPosX;

    /// <summary>
    /// Верхняя координата прорисовки бензовоза
    /// </summary>
    protected int? _startPosY;

    /// <summary>
    /// Ширина прорисовки бензовоза
    /// </summary>
    private readonly int _drawingTruckWidth = 105; 

    /// <summary>
    /// Высота прорисовки бензовоза
    /// </summary>
    private readonly int _drawingTruckHeight = 70; 

    /// <summary>
    /// Координата Х объекта
    /// </summary>
    public int? GetPosX => _startPosX;

    /// <summary>
    /// Координа У объекта
    /// </summary>
    public int? GetPosY => _startPosY;

    /// <summary>
    /// Ширина прорисовки воза(грузовика)
    /// </summary>
    public int GetWidth => _drawingTruckWidth;

    /// <summary>
    /// Высота прорисовки воза(грузовика) 
    /// </summary>
    public int GetHeight => _drawingTruckHeight;

    /// <summary>
    /// Пустой конструктор
    /// </summary>

    private DrawingTruck()
    {
        _pictureWidth = null;
        _pictureHeight = null;
        _startPosX = null;
        _startPosY = null;
    }
    /// <summary>
    /// конструктор
    /// </summary>
    /// <param name="speed">Скорость</param>
    /// <param name="weight">Вес</param>
    /// <param name="bodyColor">Основной цвет</param>

    public DrawingTruck(int speed, double weight, Color bodyColor) : this()
    {
        EntityTruck = new EntityTruck(speed, weight, bodyColor);

    }
    /// <summary>
    /// Конструктор для наследников
    /// </summary>
    /// <param name="drawingTruckWidth">Ширина прорисовки грузовика</param>
    /// <param name="drawingTruckHeight"> Высота прорисовки грузовика</param>

    protected DrawingTruck(int drawingTruckWidth, int drawingTruckHeight) : this()
    {
        _drawingTruckWidth = drawingTruckWidth;
        _drawingTruckHeight = drawingTruckHeight;

    }
    /// <summary>
    /// Установка границ поля
    /// </summary>
    /// <param name="width">Ширина поля</param>
    /// <param name="height">Высота поля</param>
    /// <returns></returns>
    public bool SetPictureSize(int width, int height)
    {
        // Проверка "влезает" ли объект в размеры поля
        if (_drawingTruckWidth > width || _drawingTruckHeight > height)
        {
            return false;
        }
        _pictureWidth = width;
        _pictureHeight = height;

        if (_startPosX.HasValue || _startPosY.HasValue)
        {
            if (_drawingTruckWidth + _startPosX > _pictureWidth)
            {
                _startPosX = _pictureWidth.Value - _drawingTruckWidth;
            }
            else if (_startPosX < 0)
            {
                _startPosX = 0;
            }
            if (_drawingTruckHeight + _startPosY > _pictureHeight)
            {
                _startPosY = _pictureHeight.Value - _drawingTruckHeight;
            }
            else if (_startPosY < 0)
            {
                _startPosY = 0;
            }
        }
        return true;
    }

    /// <summary>
    /// Установка позиции
    /// </summary>
    /// <param name="x">Координата х</param>
    /// <param name="y">Координата у</param>

    public void SetPosition(int x, int y)
    {
        if (!_pictureHeight.HasValue || !_pictureWidth.HasValue)
        {
            return;
        }
        if (x >= 0 && x + _drawingTruckWidth <= _pictureWidth && y >= 0 && y + _drawingTruckHeight <= _pictureHeight)
        {
            _startPosX = x;
            _startPosY = y;

        }
        if (x < 0)
        {
            _startPosX = 0;
        }
        if (y < 0)
        {
            _startPosY = 0;
        }
        if (x + _drawingTruckWidth > _pictureWidth)
        {
            _startPosX = _drawingTruckWidth - _pictureWidth;
        }
        if (y + _drawingTruckHeight > _pictureHeight)
        {
            _startPosY = _drawingTruckHeight - _pictureHeight;
        }
    }
    /// <summary>
    /// Изменение направления
    /// </summary>
    /// <param name="direction">Направление</param>
    public bool MoveTransport(DirectionType direction)
    {
        if (EntityTruck == null || !_startPosX.HasValue || !_startPosY.HasValue)
        {
            return false;

        }
        switch (direction)
        {
            // влево
            case DirectionType.Left:
                if (_startPosX.Value - EntityTruck.Step > 0)
                {
                    _startPosX -= (int)EntityTruck.Step;
                }
                return true;
            // вверх
            case DirectionType.Up:
                if (_startPosY.Value - EntityTruck.Step > 0)
                {
                    _startPosY -= (int)EntityTruck.Step;
                }
                return true;
            // вправо
            case DirectionType.Right:
                if (_startPosX.Value + EntityTruck.Step + _drawingTruckWidth < _pictureWidth)
                {
                    _startPosX += (int)EntityTruck.Step;
                }
                return true;
            // вниз
            case DirectionType.Down:
                if (_startPosY.Value + EntityTruck.Step + _drawingTruckHeight < _pictureHeight)
                {
                    _startPosY += (int)EntityTruck.Step;
                }
                return true;
            default:
                return false;

        }
    }

    /// <summary>
    /// Прорисовка объекта
    /// </summary>
    /// <param name="g"></param>
    public virtual void DrawTransport(Graphics g)
    {
        if (EntityTruck == null || !_startPosX.HasValue || !_startPosY.HasValue)
        {
            return;
        }
        Pen pen = new(Color.Black);


        // колеса
        Brush brBlack = new SolidBrush(Color.Black);
        g.FillEllipse(brBlack, _startPosX.Value + 5, _startPosY.Value + 50, 20, 20);
        g.FillEllipse(brBlack, _startPosX.Value + 25, _startPosY.Value + 50, 20, 20);
        g.FillEllipse(brBlack, _startPosX.Value + 85, _startPosY.Value + 50, 20, 20);

        // нижняя платформа
        Brush brBody = new SolidBrush(EntityTruck.BodyColor);
        g.FillRectangle(brBody, _startPosX.Value + 5, _startPosY.Value + 40, 100, 10);
        g.DrawRectangle(pen, _startPosX.Value + 5, _startPosY.Value + 40, 100, 10);


        // кабина
        Brush brLightBlue = new SolidBrush(Color.LightBlue);
        g.FillRectangle(brLightBlue, _startPosX.Value + 80, _startPosY.Value + 15, 25, 25);
        g.DrawRectangle(pen, _startPosX.Value + 80, _startPosY.Value + 15, 25, 25);

    }

}
