﻿using ProjectGasolineTanker.Drawings;
namespace ProjectGasolineTanker.MovementStrategy;
/// <summary>
/// Класс-реализация IMoveableObject с использованием DrawingTruck
/// </summary>
public class MoveableTruck : IMoveableObject
{
    /// <summary>
    /// Поле-объект класса DrawingTruck или его наследника
    /// </summary>
    private readonly DrawingTruck? _truck = null;

    /// <summary>
    /// Конструктор
    /// </summary>
    /// <param name="truck">Объект класса DrawingTruck</param>
    public MoveableTruck(DrawingTruck truck)
    {
        _truck = truck;
    }

    public ObjectParameters? GetObjectPosition
    {
        get
        {
            if (_truck == null || _truck.EntityTruck == null || !_truck.GetPosX.HasValue || !_truck.GetPosY.HasValue)
            {
                return null;
            }
            return new ObjectParameters(_truck.GetPosX.Value, _truck.GetPosY.Value, _truck.GetWidth, _truck.GetHeight);
        }
    }

    public int GetStep => (int)(_truck?.EntityTruck?.Step ?? 0);

    public bool TryMoveObject(MovementDirection direction)
    {
        if (_truck == null || _truck.EntityTruck == null)
        {
            return false;
        }
        return _truck.MoveTransport(GetDirectionType(direction));
    }

    /// <summary>
    /// Конвертация из MovementDirection в DirectionType
    /// </summary>
    /// <param name="direction"></param>
    /// <returns></returns>
    private static DirectionType GetDirectionType(MovementDirection direction)
    {
        return direction switch
        {
            MovementDirection.Left => DirectionType.Left,
            MovementDirection.Right => DirectionType.Right,
            MovementDirection.Up => DirectionType.Up,
            MovementDirection.Down => DirectionType.Down,
            _ => DirectionType.Unknow,

        };
    }
}
